
module Tomte
  module Document
    module Extensibility
      extend ActiveSupport::Concern

      included do
        key :_extensible, Boolean, :default => true
      end

      module ClassMethods

        def inextensible!
          key :_extensible, Boolean, :default => false
        end
      end

      def inextensible!
        self[:_extensible] = false
      end

      def extensible?
        self[:_extensible]
      end

      def method_missing(mid, *args) # :nodoc:
        raise(NoMethodError, "undefined method `#{mid}' for #{self}", caller(1)) if !extensible?

        mname = mid.id2name
        len = args.length
        if mname.chomp!('=') && mid != :[]=
          if len != 1
            raise ArgumentError, "wrong number of arguments (#{len} for 1)", caller(1)
          end
          self[mname] = args[0]
        elsif len == 0 && mid != :[]
          self[mid]
        else
          raise NoMethodError, "undefined method `#{mid}' for #{self}", caller(1)
        end
      end

    end
  end
end
