# encoding: UTF-8

module Tomte
  module Document
    module Serialization
      extend ActiveSupport::Concern

      included do
        class_attribute :include_root_in_json
        self.include_root_in_json = false
      end


      def serializable_attributes
        attributes.keys.map { |k| k.to_s } + ['id'] - ['_id']
      end

      def serializable_hash(options = nil)
        options ||= {}

        options[:only]   = Array.wrap(options[:only]).map { |k| k.to_s }
        options[:except] = Array.wrap(options[:except]).map { |k| k.to_s }

        attribute_names = serializable_attributes

        if options[:only].any?
          attribute_names &= options[:only]
        elsif options[:except].any?
          attribute_names -= options[:except]
        end

        attribute_names += Array.wrap(options[:methods]).map { |m| m.to_s }.select do |method|
          respond_to?(method)
        end

        hash = attribute_names.sort.inject({}) do |hash, name|
          value = send(name)
          hash[name] = if value.is_a?(Array)
            value.map {|v| v.respond_to?(:serializable_hash) ? v.serializable_hash : v }
          elsif value.respond_to?(:serializable_hash)
            value.serializable_hash
          else
            value
          end
          hash
        end

        hash
      end


      def as_json(options = nil)
        hash = serializable_hash(options)

        if include_root_in_json
          custom_root = options && options[:root]
          hash = { custom_root || self.class.model_name.element => hash }
        end

        hash
      end

      private


      module ClassMethods
        def from_json(json)
          self.new.from_json(json)
        end

      end
    end

  end
end
